/************************************************************************************\
 * -> Implementation of kernel binary queue data structure                          *
 * - TODO: test how it works with queues, that holds only fixed values              *
 * - TODO: implement functionality for queues with determined but different block   *
 *   size                                                                           *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#include <klib/kbqueue.h>
#include <klib/types.h>
#include <arch/string.h>
#include <arch/types.h>

static bool __insert_delim(kbqueue*, kdb_t*);
static int __get_els_delim(kbqueue*, kdb_t*, const int);
static size_t __free_size_delim(kbqueue*, const size_t, const bool);
static bool __insert_fixed(kbqueue*, kdb_t*);
static int __get_els_fixed(kbqueue*, kdb_t*, const int);
static bool __free_size_fixed(kbqueue*, const size_t, const bool);
static void __update_offset(kbqueue*, const size_t, const size_t, const size_t);

size_t kbq_get_csize(const kbqueue* q)
{
  return q->cur_size;
}

kbq_state kbq_get_state(const kbqueue* q)
{
  kbq_state state;
  
  if(q->cur_size == 0)
	state = KBQ_EMPTY;
  else if(q->cur_size == q->max_size)
	state = KBQ_FULL;
  else
	state = KBQ_PRTF;

  return state;
}

size_t kbq_insert(kbqueue* q, const kdb_t* b)
{
  if((b->size == 0) || (b->data == NULL))
	return -1;

  if(q->flags & KBQ_BNDT)
	return __insert_bndt(q, b);
  if(q->flags & KBQ_BFIXED)
	return __insert_bfixed(q, b);

  return -1;
}

bool kbq_get(kbqueue* q, kdb_t* b)
{
  if(q->cur_size == 0)
	return false;
  if(q->flags & KBQ_BNDT)
	return __get_bndt(q, b);

  return __get_fixed(q, b);
}

size_t kbq_free_space(kbqueue* q, const size_t size)
{
  if(q->flags & KBQ_BNDT)
	return __free_space_bndt(q, size + sizeof(size));

  return __free_space_bfixed(q, size);
}

static size_t __insert_bndt(kbqueue* q, const kdb_t* b)
{
  size_t writed = b->size + sizeof(b->size);

  if((b->size + sizeof(size_t)) > (q->max_size - q->cur_size)) {
	if(q->flags & KBQ_SSTRICT)
	  return -1;

	writed = __free_space_bndt(q, writed);
  }

  memmove(q->space + q->offset, &b->size, sizeof(b->size));
  memmove(q->space + q->offset + sizeof(b->size), b->data, b->size);
  q->cur_size += writed;

  if(q->flags & KBQ_GMOVE)
	__update_offset(q, writed, 0);

  return writed;
}

static bool __get_bndt(kbqueue* q, kdb_t* b)
{
  size_t len;
  
  if(q->flags & KBQ_GMOVE) {
	if(q->offset == q->cur_size)
	  return false;

	memmove(&b->size, q->space + q->offset, q->cur_size - q->offset);
	memmove(b->data, q->space + q->offset + sizeof(size_t), b->size);
	__update_offset(q, b->size + sizeof(size_t), 0);

	return true;
  }

  len = q->cur_size;
  memmove(&b->size, q->space, sizeof(size_t));
  memmove(b->data, q->space + sizeof(size_t), b->size);

  if((len -= sizeof(size_t) + b->size) == 0)
	len = 0;
  else {
	memmove(q->space, q->space + sizeof(size_t) + b->size, len);
	memset(q->space + len, 0, q->cur_size - len);
  }

  q->cur_size = len;
  return true;
}

static size_t __free_space_bndt(kbqueue* q, const size_t size)
{
  size_t bs, ns, len = q->cur_size;
  char* space = q->space;

  if(b->size > q->max_size)
	return -1;
  if(size <= (q->max_size - q->cur_size))
	return size;
  
  bs = ns = 0;

  while(ns < size) {
	memmove(&bs, space, sizeof(size_t));
	ns += bs + sizeof(size_t);
	space += ns;
  }

  if((len -= ns) == 0)
	len = 0;
  else {
	memmove(q->space, q->space + ns, len);
	memset(q->space + len, 0, q->cur_size - len);
  }
  
  q->cur_size = len;

  if(q->flags & KBQ_GMOVE)
	__update_offset(q, 0, ns);
  
  return ns;
}

static size_t __insert_bfixed(kbqueue* q, const kdb_t* b)
{
  size_t writed = 0;
  
  if(b->size > (q->max_size - q->cur_size)) {
	if(q->flags & KBQ_SSTRICT)
	  return -1;
	if(__free_space_bfixed(q, b->size) < 0)
	  return -1;
  }

  memmove(q->space + q->offset, b->data, b->size);
  q->cur_size += b->size;

  if(q->flags & KBQ_GMOVE)
	__update_offset(q, b->size, 0);

  return b->size;
}

static bool __get_bfixed(kbqueue* q, kdb_t* b)
{
  b->size = q->block_size;
   
  if(q->flags & KBQ_GMOVE) {
	if(q->offset == q->cur_size)
	  return false;

	memmove(b->block, q->space + q->offset, b->size);
	__update_offset(q, b->size, 0);

	return true;
  }

  if((q->cur_width -= b->size) < 0)
	q->cur_width = 0;
  else {
	memmove(b->block, q->space, b->size);
	memmove(q->space, q->space + b->size, q->widht);
  }

  return true;
}

static size_t __free_bfixed(kbqueue* q, const size_t size)
{
  size_t ns = 0, len = q->cur_size;
  char* space = q->space;

  if(size > q->max_size)
	return -1;
  if(size <= (q->max_size - q->cur_size))
	return size;

  ns = (((size % q->block_size) == 0) ?
		size / q->block_size : size / q->block_size + 1);

  if((len -= ns) == 0)
	len = 0;
  else
	memmove(q->space, q->space + ns, len);

  memset(q->space + len, 0, q->cur_size - len);
  q->cur_size = len;

  if(q->flags & KBQ_GMOVE)
	__update_offset(q, 0, ns);

  return ns;
}

static void __update_offset(kbqueue* q, const size_t bw, const size_t bd)
{
  q->offset += bw;

  if((q->offset -= bd) < 0)
	q->offset = 0;
}

