/************************************************************************************\
 * Kernel Output System (kos)                                                       *
 * > ikernel/eza/klib/kos.c                                                         *
 * - kernel printting system                                                        *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#include <klib/kio.h>
#include <klib/klib.h>
#include <klib/varargs.h>
#include <klib/types.h>
#include <video/kconsole.h>
#include <arch/string.h>
#include <arch/types.h>

#define TMPBUF_SIZE  1024
#define KERR_MSG      "ERROR: "
#define KERR_MSG_LEN  7
#define KWARN_MSG     "WARNING: "
#define KWARN_MSG_LEN 9

static kbuffer kbuf = {
  .cur_size = 0,
  .size = KBUF_SIZE
};

kconsole _kcons;

size_t kbuf_insert(const char* s, const size_t slen)
{
  if(slen > kbuf.size)
	return -1;
  if(slen > (kbuf.size - kbuf.cur_size)) {
	memset(&kbuf, '\0', kbuf.cur_size);
	kbuf.cur_size = 0;
  }

  strncpy(kbuf.space + kbuf.cur_size, s, slen);
  kbuf.cur_size += slen;

  return slen;
}


char* kbuf_get(void)
{
  if(kbuf.cur_size == 0)
	return NULL;

  return kbuf.space + kbuf.cur_size;
}

void kprintf(const char* fmt, ...)
{
  va_list ap;
  
  va_start(ap, fmt);
  vkfprintf(KOS_NORMAL, fmt, ap);
  va_end(ap);
}

void vkprintf(const char* fmt, va_list ap)
{
  vkfprintf(KOS_NORMAL, fmt, ap);
}

void kfprintf(const uint8_t flags, const char* fmt, ...)
{
  va_list ap;

  va_start(ap, fmt);
  vkfprintf(flags, fmt, ap);
  va_end(ap);
}

void vkfprintf(const uint8_t flags, const char* fmt, va_list ap)
{
  char tmp_buf[TMPBUF_SIZE];
  size_t sl, pref_len;

  memset(tmp_buf, '\0', TMPBUF_SIZE);

  if(flags & KOS_ERROR) {
	strncpy(tmp_buf, KERR_MSG, KERR_MSG_LEN);
	pref_len = KERR_MSG_LEN;
  }
  else if(flags & KOS_WARNING) {
	strncpy(tmp_buf, KWARN_MSG, KWARN_MSG_LEN);
	pref_len = KWARN_MSG_LEN;
  }
  else {
	pref_len = 0;
  }

  sl = vsprintf(tmp_buf + pref_len, fmt, ap);

  if((sl += pref_len) > TMPBUF_SIZE) {
	_kcons.display_string("\nvsnprintf error\n");
	return;
  }
  if(kbuf_insert(tmp_buf, sl) != sl) {
	_kcons.display_string("\nkbuf error\n");
	return;
  }
  if(_kcons.is_enable())
	_kcons.display_string(tmp_buf);
}

