/************************************************************************************\
 * > ikernel/eza/video/kconsole.c                                                   *
 * - Jari is a microkernel, so all drivers should                                   *
 * - implement like kernel servers. But we can't use servers                        *
 * - before we init sheduler and memory manager. But we want                        *
 * - print on screen some mjor system things. So, in this case                      *
 * - we implement simple vga console in the kernel                                  *
 * - for printing on screen only.                                                   *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#include <config.h>
#include <klib/types.h>
#include <video/kconsole.h>
#include <video/vga.h>
#include <arch/string.h>
#include <arch/types.h>

static void kcons_enable(void);
static void kcons_disable(void);
static void kcons_greeting(void);
static void kcons_display_string(const char*);
static void kcons_display_char(const char);
static bool kcons_is_enable(void);

#ifdef KCONS_COLOR_MODE
static bool __is_warn_or_err(const char*, const uint8_t);
static void __display_warn_or_err(const uint8_t);
#endif /* KCONS_COLOR_MODE */

kconsole _kcons = {
  .enable = kcons_enable,
  .greeting = kcons_greeting,
  .is_enable = kcons_is_enable,
  .display_string = kcons_display_string,
  .display_char = kcons_display_char,
  .disable = kcons_disable,
  .__enable = false,
};

static void kcons_enable()
{
  vga_init();
  vga_cursor(ENABLE);
  //  vga_set_cursor_attrs(VGA_CRSR_LINE);
  vga_set_bg(KCONS_DEF_BG);
  vga_set_fg(KCONS_DEF_FG);  
  vga_cls();
  _kcons.__enable = true;
}

static void kcons_disable()
{
  _kcons.__enable = false;
}

static bool kcons_is_enable()
{
  return _kcons.__enable;
}

static void kcons_greeting()
{
  char greet[11] = "Jari kernel";
  uint16_t x;
  int i;

  x = vga_get_cols();
  x = x / 2 - 11;
  vga_set_x(x);
  vga_set_fg(GREEN);

  for(i = 0; i < 11; i++)
	vga_putch(greet[i]);

  vga_putch('\n');
  vga_putch('\n');
  vga_set_fg(WHITE);
}

static void kcons_display_string(const char* str)
{
#ifdef KCONS_COLOR_MODE
  /* insert some code here */  
#endif /* KCONS_COLOR_MODE */

  char* p;
  
  for(p = str; *p != '\0'; p++)
	vga_putch(*p);

  vga_update_cursor();
}

static void kcons_display_char(const char c)
{
  vga_putch(c);
  //  vga_update_cursor();
}

#ifdef KCONS_COLOR_MODE

static bool __is_warn_or_err(const char* str, const uint8_t type)
{
  char *s, *pattern;
  /*
  if(type & KWARN)
	pattern = KWARN_MSG;
  else
  pattern = KERR_MSG;
  */

  if((s = strstr(str, pattern)) && (s == str))
	return true;

  return false;
}

static void __display_warn_or_err(const uint8_t type)
{
  char* msg;
  uint16_t save_bg, save_fg;

  save_bg = vga_get_bg();
  save_fg = vga_get_fg();
  /*
  if(type & KWARN) {
	msg = KWARN_MSG;
	vga_set_bg(KCONS_WARN_BG);
	vga_set_fg(KCONS_WARN_FG);
  }
  else {
	msg = KERR_MSG;
  	vga_set_bg(KCONS_ERR_BG);
	vga_set_fg(KCONS_ERR_FG);
	}
  */

  for(; *msg != '\0'; msg++)
	vga_putch(*msg);

  vga_set_bg(save_bg);
  vga_set_fg(save_fg);
}

#endif /* KCONS_COLOR_MODE */
