/************************************************************************************\
 * > ikernel/eza/video/vesavga.c                                                    *
 * - VESA/VGA basic functionality                                                   *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#include <video/vga.h>
#include <arch/io.h>
#include <arch/string.h>
#include <arch/types.h>

static vga_dev vga;
static vga_regs vregs;

void vga_set_x(const uint16_t x)
{
  vga.x = x;
}

void vga_set_y(const uint16_t y)
{
  vga.y = y;
}

void vga_set_fg(const uint16_t fg)
{
  vga.fg = fg;
}

void vga_set_bg(const uint16_t bg)
{
  vga.bg = bg;
}

uint16_t vga_get_cols()
{
  return vga.cols;
}

uint16_t vga_get_rows()
{
  return vga.rows;
}

uint16_t vga_get_fg()
{
  return vga.fg;
}

uint16_t vga_get_bg()
{
  return vga.bg;
}

uint16_t vga_get_x()
{
  return vga.x;
}

uint16_t vga_get_y()
{
  return vga.y;
}

inline void vga_make_attr(uint16_t* out)
{
  *out |= (vga.fg << 8) | (vga.bg << 12);
}

void vga_init(void)
{
  /* TODO: add logic, that choice needed mode */
  
  vga.base = (uint16_t*)VGA_TM_BASE;
  vga.cols = 80;
  vga.rows = 25;
  vga.x = vga.y = 0;
  memset(&vregs, 0, sizeof(vregs));
  vga.es_char = -1;

  /* configure _available_ write mode */
  
}

void vga_cls()
{
  uint16_t out;

  out = SPACE;
  vga_make_attr(&out);

  memset(vga.base, out, vga.cols * vga.rows * 2);
  vga.x = vga.y = 0;
  vga_update_cursor();
}

void vga_update_cursor()
{
  uint16_t where, crt_dp, crt_ap;
  uint16_t crt_ar;

  where =  vga.y * vga.cols + vga.x - 1;
  crt_dp = VGA_SET_MODE(VDP_CRT, VGA_COLOR); /* CRT data port */
  crt_ap = VGA_SET_MODE(VAP_CRT, VGA_COLOR); /* CRT address port */

  crt_ar = inb(crt_ap);
  outb(crt_ap, CLHR);
  vregs.crtc[CLHR] = where >> 8;
  outb(crt_dp, vregs.crtc[CLHR]);
  outb(crt_ap, CLLR);
  vregs.crtc[CLLR] = where & 0xFFFFFFFF;
  outb(crt_dp, vregs.crtc[CLLR]);
  outb(crt_ap, crt_ar);
}

void vga_cursor(const bool action)
{
  uint16_t crt_dp, crt_ap, crt_ar;

  crt_dp = VGA_SET_MODE(VDP_CRT, VGA_COLOR);
  crt_ap = VGA_SET_MODE(VAP_CRT, VGA_COLOR);

  crt_ar = inb(crt_ap);
  outb(crt_ap, CSR);
  vregs.crtc[CSR] = inb(crt_dp);

  if(!action && !(vregs.crtc[CSR] & VGA_CDISABLE))
	outb(crt_dp, vregs.crtc[CSR] | VGA_CDISABLE);
  if(action && (vregs.crtc[CSR] & VGA_CDISABLE))
	outb(crt_dp, vregs.crtc[CSR] & VGA_CENABLE);

  outb(crt_ap, crt_ar);
}

void vga_set_cursor_attrs(const uint8_t flags)
{
  uint16_t crt_dp, crt_ap, crt_ar;

  crt_dp = VGA_SET_MODE(VDP_CRT, VGA_COLOR);
  crt_ap = VGA_SET_MODE(VAP_CRT, VGA_COLOR);
  crt_ar = inb(crt_ap);
  /*outb(crt_ap, MSLR);
  vregs.crtc[MSLR] = inb(crt_dp);
  outb(crt_dp, vregs.crtc[MSLR] | (1 & 0x20));*/

  if(flags & VGA_CRSR_LOW) {
	vga_putch('C');
    outb(crt_ap, CSR);
	vregs.crtc[CSR] = inb(crt_dp);
	outb(crt_dp, 0x0A);
  }
  if(flags & VGA_CRSR_HIGH) {
	vga_putch('G');
	outb(crt_ap, CER);
	vregs.crtc[CER] = inb(crt_dp);
	outb(crt_dp, 0x0B);
  }

  outb(crt_ap, crt_ar);
}

void vga_blinking(const bool action)
{
  uint16_t isr1_dp, isr1_save, atr_save;

  isr1_dp = VGA_SET_MODE(VDP_ISR1, VGA_COLOR);
  isr1_save = inb(isr1_dp);
  atr_save = inb(VDP_ATR_RD);
  outb(VAP_ATR_WR, AMCR);
  vregs.atr[AMCR] = inb(VDP_ATR_RD);

  if(action && !(vregs.atr[AMCR] & VGA_ABLINK_ENABLE))
	outb(VAP_ATR_WR, vregs.atr[AMCR] | VGA_ABLINK_ENABLE);
  if(!action && (vregs.atr[AMCR] & VGA_ABLINK_ENABLE))
	outb(VAP_ATR_WR, vregs.atr[AMCR] & VGA_ABLINK_DISABLE);

  outb(VAP_ATR_WR, atr_save);
  outb(isr1_dp, isr1_save);
}

void vga_putch(uint16_t c)
{
  uint16_t *where;
  
  switch(c) {
  case BACKSPACE:
	if(vga.x != 0)
	  vga.x--;

	break;
  case TAB:
	vga.x += 8 & ~(8 - 1);
	break;
  case '\r':
	vga.x = 0;
	break;
  case '\n':
	vga.x = 0; /* increment to 0 later */
	vga.y++;
	break;
  default:
	if(c < ' ')
	  break;

	vga_make_attr(&c);
	where = vga.base + vga.cols * vga.y + vga.x;
	*where = c;
	vga.x++;
	vga_scroll();
  }
}

void vga_scroll(void)
{
  int i;
  uint16_t diff, *where, out;
  
  if(vga.x >= vga.cols) {
	diff = (vga.x - vga.cols) % vga.cols;
	vga.y++;
	vga.x = diff;
  }
  if(vga.y >= vga.rows) {
	vga.y = vga.rows - 2;
	out = SPACE;
	
	vga_make_attr(&out);
	memmove(vga.base, vga.base + 2 * vga.cols, vga.cols * (vga.rows - 1) * 2);
	memset(vga.base + vga.cols * (vga.rows - 1), out, vga.cols * 2 + 1);
  }
}

