/************************************************************************************\
 * - Calls main kernel function and inits some major assembler functions            *
 * - We initialize paging before proccessor vectors, because our kernel             *
 *   linked to virtual address 0xC0000000.  And we can't init proccessor            *
 *   vectors without paging in this case...                                         *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

/*
TODO:
	1) clear bss (not so important as i mentioned)
	2) unreg interrupts
*/

.text
	
#include <boot/boot.h>
#include <arch/page.h>
	
.code32
.align 4
.globl _start, setup

multiboot_header:
        .long MULTIBOOT_MAGIC
	.long MULTIBOOT_FLAGS
        .long MULTIBOOT_CHECKSUM

.set KERNEL_START_PHYS, 0x100000
.set KERNEL_START_VIRT, 0xC0000000
.set STACK_SIZE, 0x4000
.set SUBSTRACT, 0x40100000
.comm stack, STACK_SIZE, 32

_start:
        cld
	/* config some major things for enabling paging */
	movl %ebx, (_mi + SUBSTRACT)
	pushl %eax
	pushl %ebx
	
	/* mapping of first 8M.
	   we create ealy boot page directory
	   and two page entrys:
	   1st - identity map
	   2nd - kernel map */

	/* FIXME: obligatory check if we can set PAE or not!!! */
	/* set on PAE and PSE flags in cr4 */
	movl $(PGCR4_PAE | PGCR4_PSE), %eax
	movl %cr4, %edx
	andl %edx, %eax
	movl %edx, %cr4
	
	/* init kernel map map page table. */
	xorl %eax, %eax
	movl $EB_KPTE, %edi
	movl $(KERNEL_START_PHYS | PGD_PRESENT | PGD_WRITABLE), %eax
	movl $(PAGE_SIZE >> 2), %ecx

map_ktable:
	stosl
	addl $PAGE_SIZE, %eax
	loop map_ktable

	/* init identyty mao page table */
	xorl %eax, %eax
	movl $(PGD_PRESENT | PGD_WRITABLE), %eax
	movl $EB_IPTE, %edi
	movl $(PAGE_SIZE >> 2), %ecx

map_itable:
	stosl
	addl $PAGE_SIZE, %eax
	loop map_itable
	
	/* create page directory */
	movl $(EB_IPTE | PGD_PRESENT | PGD_WRITABLE), EB_KPDE
	movl $(EB_KPTE | PGD_PRESENT | PGD_WRITABLE), EB_KPDE + (KERNEL_START_VIRT >> PAGE_SHIFT) / (PAGE_SIZE >> 2) * 4
	movl $(EB_KPDE | PGD_PRESENT | PGD_WRITABLE), EB_KPDE + PAGE_SIZE - 4	

	/* enable paging */
	movl $EB_KPDE, %eax
	movl %eax, %cr3
	movl %cr0, %eax
	orl $0x80010000, %eax
	movl %eax, %cr0
	
//	movl sys_stack, %esp
	movl $(stack + STACK_SIZE), %esp

	/* parse and save multiboot info structure */
	call kernel_init /* main kernel function */

.data
.align 0x1000
kpde:
	.space 0x1000, 0
kide:
	.space 0x1000, 0
	
