/************************************************************************************\
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/


#include <arch/cpu.h>

/* FIXME: later move it to interrupts.h */
#define SYSCALL_VECTOR 0xAD

static idt_entry idt[IDT_SIZE];

static inline void __load_idt(tdesc_ptr* idt_ptr)
{
  __asm__ volatile ("lidt %0\n" : : "m" (*idt_ptr));
}

/*******************************************************
  basic initialisation IDT vectors.                    *
  bassically occurs before trap handlers initalisation *
                                                      */
void idt_init()
{
  tdesc_ptr idt_ptr;
  int i;

  idt_ptr.limit = sizeof(idt);
  idt_ptr.base = (unsigned long)idt;

  __load_idt(&idt_ptr);

  for(i = 0; i < IDT_SIZE; i++) {
	idt[i].offset_low = idt[i].offset_high = 0;
	idt[i].unused = idt[i].selector = 0;

	if(i == SYSCALL_VECTOR)
	  idt[i].access = DPL_USER;
	else
	  idt[i].access = DPL_KERNEL;

	idt[i].access |= STF_PRESENT;
  }
}

void idt_set_offset(idt_entry* idte, const uint32_t offset)
{
  idte->offset_low = 0x0000FFFF & offset;
  idte->offset_high = 0xFFFF0000 & offset;
}
