/************************************************************************************\
 * > ikernel/include/boot/boot.h                                                    *
 * - basic stuff for multiboot                                                      *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#ifndef __BOOT_H__
#define __BOOT_H__

#ifndef __ASM__
#include <arch/types.h>
#endif /* __ASM__ */

/* start of multiboot defenitions */
#define MULTIBOOT_MAGIC 0x1BADB002

/* turn on memory info and align loaded modules on page boundaries */
#define MULTIBOOT_FLAGS 0x00000003
#define MULTIBOOT_CHECKSUM -(MULTIBOOT_MAGIC + MULTIBOOT_FLAGS)
/* end of multiboot defenitions */

#ifndef __ASM__

/* multiboot header structure */
typedef struct mb_header {
  uint32_t magic;
  uint32_t flags;
  uint32_t checksum;
  /* address corresponding to begin of multiboot header */
  uint32_t header_addr;
  /* physical address of the beginning of the text segment */
  uint32_t load_addr;
  /* physical address of the end of the data segment */
  uint32_t load_end_addr;
  /* physical address of the end of the bss segment */
  uint32_t bss_end_addr;
  /* physical address at which operation system runs */
  uint32_t entry_addr;
  
  /**************************
	0 - linear graphic mode *
    1 - EGA text mode       *
                           */
  uint32_t mode_type;
  uint32_t width;  /* number of the columnt */
  uint32_t height; /* number of the lines */
  uint32_t depth;  /* number of the bits per pixel */
} mb_header;

/* multiboot info structure */
typedef struct mb_info {
  uint32_t flags;
  uint32_t mem_lower;     /* amount of lower memory 0M - 640M */
  uint32_t mem_upper;     /* amount of upper memory 1G - NG */
  uint32_t boot_device;   /* indicates BIOS disk device */
  uint32_t cmdline;       /* physical address of the passed kernel comand line */
  uint32_t mods_count;    /* number of loaded modules */
  uint32_t mods_addr;     /* physical address of first module structure */
  uint32_t syms[4];     
  uint32_t mmap_length;   /* size of memory map buffer */
  uint32_t mmap_addr;     /* physical address of memory map buffer */
  uint32_t drives_length; /* size of drive structure */
  uint32_t drives_addr;   /* physical address of the first drive structure */
  uint32_t config_table;  /* physical address of ROM configuration table */
  uint32_t boot_loader_name;
  uint32_t apm_table;     /* physical address of APM table */
  /* some video stuff */
  uint32_t vbe_control_info;
  uint32_t vbe_mode_info;
  uint16_t vbe_mode;
  uint16_t vbe_interface_seg;
  uint16_t vbe_interface_off;
  uint16_t vbe_interface_len;
} mb_info;

extern mb_info* _mi;

#endif /* __ASM__ */
#endif /* __BOOT_H__ */
