/*
 * JP (Jari Protocol) for internal and external,
 * kernel and/or server communication.
 * This is a part of the Jari microkernel.
 * 
 * Cmd subpart.
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

/*
 * Cmd frame contains the following structure:
 * /=========================================================\
 * | Flag               <-- unique flag for the frame        |
 * |---------------------------------------------------------|
 * | Cmd                <-- command id                       |
 * |---------------------------------------------------------|
 * | Nargs              <-- number of the args for speed up  |
 * |---------------------------------------------------------|
 * | Args_offset        <-- offset of the whole data         |
 * |---------------------------------------------------------|
 * | Args_length        <-- length of the whole data         |
 * |---------------------------------------------------------|
 * | Aoff0              <-- offset of the 0th arg            |
 * |---------------------------------------------------------|
 * | Alegth0            <-- length of the 0th arg            |
 *                              ....
 * | Aoffn              <-- offset of the nth arg            |
 * |---------------------------------------------------------|
 * | Alegthn            <-- length of the nth arg            |
 * \=========================================================/
 */

#define CMD_FLAG 0x5E

typedef struct __cmd_header {
  char flag;
  char cmd; /*id of the command*/
  int nargs; /*number of the arguments*/
  unsigned long args_offset; /*global frame offset*/
  unsigned long args_length; /*global frame length*/
  unsigned long *aoff; /*offsets of the arguments*/
  unsigned long *alen; /*lengths of the arguments*/
} cmd_header;

typedef struct __cmd_frame {
  cmd_header head;
  void *data; /*<--- arguments data*/
} cmd_frame;

/*general functions*/
int check_cmd_header(cmd_header *p);
int create_cmd_header(cmd_header *p,char cmd,int nargs);
int create_cmd_frame(cmd_header *p,cmd_frame *f,
		     void *data,unsigned long *lns);
int reject_cmd_frame(cmd_frame *p);
int check_cmd_frame(cmd_frame *p);

/*global args functions*/
void *get_cmd_whole_frame_data_ptr(cmd_frame *p);
int get_cmd_whole_frame_data(cmd_frame *p,void *data);
unsigned long get_ipc_whole_frame_data_length(cmd_frame *p);
unsigned long get_ipc_whole_frame_data_offset(cmd_frame *p);

/*arguments functions*/
unsigned long get_cmd_frame_arg_length(cmd_frame *p,int narg);
void *get_cmd_frame_arg_ptr(cmd_frame *p,int narg);
int get_cmd_frame_arg(cmd_frame *p,void *arg,int narg);

