/*
 * JP (Jari Protocol) for internal and external,
 * kernel and/or server communication.
 * This is a part of the Jari microkernel.
 * 
 * IPC subpart.
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

/*
 * IPC frame header contains the following structure:
 * /=========================================================\
 * | Flag               <-- unique flag for the frame        |
 * |---------------------------------------------------------|
 * | Type               <-- frame type                       |
 * |---------------------------------------------------------|
 * | ChunkID            <-- unique ID for the chunk          |
 * |---------------------------------------------------------|
 * | TrunkID            <-- unique IS for the trunk          |
 * |---------------------------------------------------------|
 * | DataOffset         <-- Data offset for the frame        |
 * |---------------------------------------------------------|
 * | DataLength         <-- Data length                      |
 * \=========================================================/
 */

#define IPC_FLAG  0x5F

/*types*/
#define STD_FRAME  0x01
#define CHUNK_BEG  0x02
#define CNUNK_END  0x03
#define TRUNK_INS  0x04
#define REQ_FRAME  0x0D

typedef struct __ipc_header {
  char flag;
  char type;
  int chunk_id;
  int trunk_id;
  unsigned long data_offset;
  unsigned long data_length;
} ipc_header;

typedef struct __ipc_frame {
  ipc_header head;
  void *data;
} ipc_frame; 

int check_ipc_header(ipc_header *p);
int create_ipc_header(ipc_header *p);
int change_ipc_frame_type(ipc_frame *p,char type);
int create_ipc_frame(ipc_header *p,ipc_frame *pt,
		     void *data,unsigned long length);
int reject_ipc_frame(ipc_frame *p);
void *get_ipc_frame_data_ptr(ipc_frame *p);
int get_ipc_frame_data(ipc_frame *p,void *data);
unsigned long get_ipc_frame_data_length(ipc_frame *p);
unsigned long get_ipc_frame_data_offset(ipc_frame *p);

