/*
 * JP (Jari Protocol) for internal and external,
 * kernel and/or server communication.
 * This is a part of the Jari microkernel.
 * 
 * IPC port subpart.
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 */

/*ports protos*/
#define PORT_SYNC  0x1
#define PORT_ASYNC 0x7
#define PORT_MONO  0xC
#define PORT_BIDI  0xE

/*port status*/
#define PORT_BUSY  0xF1
#define PORT_FROZ  0xA1
#define PORT_WAIT  0xC0

typedef struct __ipc_port_t {
  char type; /*sync,async,monodirectional,bidirectional*/
  char state; /*frozen,busy,waiting*/
  int server_id; /*id of the server where we connected*/
  int inum; /*port number*/
  unsigned long time; /*time of waiting*/
  unsigned long address; /*address*/
} ipc_port_t;

int create_ipc_port(ipc_port_t p,int port);
int set_ipc_port_state(char state);
int set_ipc_port_type(char type);
int connect_ipc_port(ipc_port_t p, int serv_id);
int recv_ipc_frame(ipc_frame *p, ipc_port_t l);
int send_ipc_frame(ipc_frame *p, ipc_port_t l);
int close_ipc_port(ipc_port_t p);

