/************************************************************************************\
 * - Some defenitions for kernel binary queues abstraction                          *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/


#ifndef __KBQUEUE_H__
#define __KBQUEUE_H__

#include <klib/types.h>
#include <arch/types.h>

/* QUEUE BLOCK TYPES */
/**********************************************************
  non determined block size, blocks sparate               *
  with some delimiter.                                    *
  beneficially use this flag for holding string in queue. *
                                                         */
#define KBQ_BNDT       0x01
/* each block has fixed determined size */
#define KBQ_BFIXED     0x02


/* QUEUE FILLING STRATAGIES */
/* if queue is full, it doesn't allow us to add new elemnts in it */
#define KBQ_SSTRICT    0x04

/******************************************************
  if queue is full and we add new element to it,      *
  queue will free needed amount of space for insering *
  new item.                                           *
                                                     */
#define KBQ_SSLIDING   0x08


/* QUEUE EXTRACTING STRATAGIES */
/* delete element after extracting it from the queue */
#define KBQ_GDEL       0x10
/********************************************
  don't delete the element after extracting *
  it from the queue,                        *
  just increment offset.                    *
                                           */
#define KBQ_GMOVE      0x20

/* kernel binary queue structure */
typedef struct kbqueue {
  void* space;     /* queue indeed */
  char flags;      /* queue flags */
  int offset;      /* offset(for sliding stratagy only) */
  size_t block_size;       /* block size (for fixed queues only) */
  size_t cur_size; /* current queue size */
  size_t max_size; /* max queue size */
} kbqueue;

/* this enum describes possuble states of queue */
typedef enum kbq_state {
  KBQ_EMPTY = 0,
  KBQ_PRTFL, /* partially full */
  KBQ_FULL
} kbq_state;

size_t kbq_insert(kbqueue*, const kdb_t*);
bool kbq_get(kbqueue*, kdb_t*);
kbq_state kbq_get_state(const kbqueue*);
size_t kbq_get_csize(const kbqueue*);

#endif /* __KBQUEUE_H__ */
