/************************************************************************************\
 * > ikernel/eza/klib/kio.h                                                         *
 * - kernel printting system                                                        *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#ifndef __KIO_H__
#define __KIO_H__

#include <config.h>
#include <klib/types.h>
#include <klib/varargs.h>
#include <arch/types.h>

/**********************************************************
  flags for kernel printing system, that identify type of *
  printing message.                                       *
                                                          *
  KTEXT - simple message, text.                           *
  KWARN - kernel warning                                  *
  KERR  - kernel error                                    *
                                                         */
#define KTEXT 0x01
#define KWARN 0x02
#define KERR  0x04

/***************************************************
  number of flags for formated I/O.                *
  all flags should fit to 16 bits.                 *
  so,                                              *
                                                   *
  - first two bits indicate type of I/O value      *
    (number/strng/character)                       *
  - next nine bits hold some number specific flags *
  - other bits hold some common modifiers          *
                                                  */
/* type of data, that will be printed (0 - 2 bits) */
#define FMT_TNUM    0x00001 /* number */
#define FMT_TSTR    0x00002 /* string */
#define FMT_TCHAR   0x00004 /* character */
/* flags */
#define FMT_FLJ     0x00008 /* '-' flag. it means, that output will be left-justigfird */
#define FMT_FZERO   0x00010 /* zero flag */
#define FMT_FSIG    0x00020 /* signed digit or just '+' flag */
#define FMT_FSPACE  0x00040 /* if first simbol of format sequince is not digit, print space. if + and space are enable, use + */
#define FMT_FPREF   0x00080 /* prefix before digit (x, X, o only) */
/* dot, just dot */
#define FMT_DOT     0x00100
/* length modifiers */
#define FMT_LCHAR   0x00200
#define FMT_LSHORT  0x00400
#define FMT_LLONG   0x00800
#define FMT_LDLONG  0x01000
#define FMT_LSIZET  0x02000
/* extension flags for digit type */
#define FMT_DOCT    0x04000 /* octal digit */
#define FMT_DDEC    0x08000 /* decimal digit */
#define FMT_DHEX    0x10000 /* hexadecimal digit */
#define FMT_DHEX_UP 0x20000 /* hexadecimal in upper case */
#define FMT_DUNSIG  0x40000 /* unsigned digit */

/* some format masks */
#define FMT_FMASK   0x00F8 /* mask for flags */
#define FMT_LMASK   0x3E00 /* mask for length modifiers */

#define FMT_DBUF_SIZE   24  /* length buffer for temprary digits holding */
#define KBQ_TMPBUF_SIZE 1024

/* kernel buffer getting stratagy */
#define KBUF_BYREF true  /* get string from kernel buffer by reference */
#define KBUF_COPY  false /* get string from kernel buffer by copying */

/* some flags for [v]kfprintf */
#define KOS_NORMAL  0x01
#define KOS_WARNING 0x02
#define KOS_ERROR   0x04

/* structure for format description */
typedef struct fattrs {
  uint32_t flags; /* format flags */
  int width;      /* format width (before dot) */
  int presiction; /* format presiction (after dot)*/
  int cinf;       /* number of characters in format */
} fattrs;

typedef struct kbuffer {
  char space[KBUF_SIZE];
  size_t cur_size;
  size_t size;
  /* TODO: add mutex field */
} kbuffer;

/* function defenitions for KOS */
void kprintf(const char*, ...);
void vkprintf(const char*, va_list);
void kfprintf(const uint8_t, const char*, ...);
void vkfprintf(const uint8_t, const char*, va_list);

/* functions, that makes format */
size_t vsprintf(char*, const char*, va_list);
size_t vsnprintf(char*, const size_t, const char*, va_list);

/* functions defenition for kernel buffer */
size_t kbuf_insert(const char*, const size_t);
char* kbuf_get(void);

#endif /* __KIO_H__ */
