/*
 * General NS structure and functions.
 * The part of the Jari microkernel.
 *
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2004, 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#define MAX_SERVERS  4096
#define MAX_PATHS    8192

/*servers types*/
#define SRV_DRIVER      0
#define SRV_DEVICE      1
#define SRV_FS          2
#define SRV_MEM         3
#define SRV_CPU         4
#define SRV_NODE        5
#define SRV_VM          6
#define SRV_MISC        7
#define SRV_CUSTOM      8
#define SRV_TERM        9
#define SRV_NET         10
#define SRV_PROTOCOL    11

#define OUT_ILLEGAL_TYPE  12

/*structures*/
typedef struct __srvchain {
  int id;
  int priority;
  int type;
  unsigned long time; /*time of the excepted execution*/
  char *name; /*name of the server*/
  void *data; /*optional data, maybe NULL*/
  struct __srvchain *next;
} srvchain;

typedef struct __route {
  int id; /*id of the server*/
  unsigned long time; /*excepted time for execution*/
  unsigned long max_size; /*reserved for some future functionality*/
  int priority; /*reserved for some future functionality*/
  struct __route *next;
} nsroute;

typedef struct __routing {
  int id;
  int priority;
  nsroute *pathroute;
  struct __routing *next;
} paths_route;

/*init functions*/
int init_nsserverschain(void); /*FIXME: temproary*/
int init_nspathsroute(void);

/*servers functions*/
int reg_server(int priority, int type, long time,
	       char *name, void *data);
int unreg_server(char *name);
int get_server_id(char *name);
int *find_server_by_type(int type);
int get_srv_priority(int id);

/*route functions*/
int add_nsroute(nsroute *p);
int get_routeid_by_ptr(nsroute *p);
int del_nsroute(nsroute *p);
int del_nsroute_by_id(int id);
int get_high_nsroute(void);

