/************************************************************************************\
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#ifndef __GDT_H__
#define __GDT_H__

#define ADDR_LIMIT 0xFFFFFFFF /* virtual address limit - 4G */
#define GDT_SIZE 5

/* GDT access flags defenition */
#define GDT_DATA (2 << 3)             /* data segment */
#define GDT_CODE (3 << 3)             /* code segment */
#define GDT_PRESENT (1 << 7)          /* segment is present */
#define GDT_PAGE_ACCESSED 1           /* page accessed */
#define GDT_PAGE_WRITABLE (1 << 1)    /* page writable */
#define GDT_PAGE_CONNFORMING (1 << 2) /* page conformed */
#define GDT_KERNEL (0 << 5)           /* kernel privelege level */
#define GDT_USER   (3 << 5)           /* user privelege level */
/* end of GDT access flags defenitions */

#ifndef __ASM__

struct gdt_entry {
  unsigned limit:16;
  unsigned base_low:16;
  unsigned base_middle:8;
  unsigned access:8;
  unsigned seg_limit:3;
  unsigned available:1;
  unsigned unused:1;
  unsigned opsize:1;
  unsigned granularity:1;
  unsigned base_high:8;
} __attribute__ ((packed));

struct gdt_ptr {
  unsigned short limit;
  unsigned int base;
} __attribute__ ((packed));

typedef struct gdt_ptr gdt_ptr;
typedef struct gdt_entry gdt_entry;

extern gdt_ptr gdtp;

void gdt_init();
void gdt_set_base(gdt_entry*, unsigned long);
void gdt_set_limit(gdt_entry*, unsigned int);
void gdt_write();
static void gdt_fill(gdt_entry*, unsigned long, unsigned long, unsigned char);

#endif /* __ASM__ */
#endif /* __GDT_H__ */
