/************************************************************************************\
 * - Implementation of in(b|w|l) and out(b|w|l) functionality                       *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#ifndef __IO_H__
#define __IO_H__

#include <arch/types.h>

static inline void outb(uint16_t port, uint8_t val)
{
  __asm__ volatile ("outb %0, %1\n"
					: : "a" (val), "Nd" (port));
}

static inline void outw(uint16_t port, uint16_t val)
{
  __asm__ volatile ("outw %0, %1\n"
					: : "a" (val), "Nd" (port));
}

static inline void outl(uint16_t port, uint32_t val)
{
  __asm__ volatile ("outl %0, %1\n"
					: : "a" (val), "Nd" (port));
}

static inline uint8_t inb(uint16_t port)
{
  uint8_t val;

  __asm__ volatile("inb %1, %0\n"
				   : "=a" (val)
				   : "d" (port));

  return val;
}

static inline uint16_t inw(uint16_t port)
{
  uint16_t val;

  __asm__ volatile("inw %1, %0\n"
				   : "=a" (val)
				   : "d" (port));

  return val;
}

static inline uint32_t inl(uint16_t port)
{
  uint32_t val;

  __asm__ volatile("inl %1, %0"
				   : "=a" (val)
				   : "d" (port));

  return val;
}

#endif /* __IO_H__ */
