/************************************************************************************\
 * > ikernel/include/arch/ll_math.h                                                 *
 * - provides basic math functions for 64 bits integers                             *
 * - for current moment can make division only [ 64 / 32 => 64 ]                    *
 * - implementation of division and mod operations for long long                    *
 * - TODO: may be i should add functions [ 64 / 64 => [32 | 64] ] ?                 *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/


#ifndef __LL_DIV_H__
#define __LL_DIV_H__

#include <video/vga.h>
#include <arch/types.h>

#define LL_MAX_SIGNED  0xFFFFFFFFFFFFFFF0LL
#define ll_getlow(ll)  (((ll) & 0xFFFFFFFF))
#define ll_gethigh(ll) (((ll) >> 32))

static inline int64_t ll_div_with_rest(int64_t divid, int32_t divis, int32_t* rest)
{
  int32_t res_lp, res_hp, rst, neg;
  int64_t res;

  res_lp = res_hp = rst = neg = 0;

  /* takes care about the sign */
  if(divid < LL_MAX_SIGNED) {
	if((divid < 0) && (divis < 0)) {
	  divid = -divid;
	  divis = -divis;
  }
	else if((divid > 0) && (divis < 0)) {
	  neg = 1;
	  divis = -divis;
  }
	else if((divid < 0) && (divis > 0)) {
	  neg = 1;
	  divid = -divid;
	}
  }
  
  __asm__ volatile("movl %4, %%ecx\n\t"
				   "testl %%ecx, %%ecx\n\t" /* if higher part is empry, make only one division */
				   "jz 1f\n\t"
				   "movl %4, %%eax\n\t"
				   "xorl %%edx, %%edx\n\t"
				   "divl %%ebx\n\t"
				   "movl %%eax, %1\n\t"    /* eax is our high 4 bytes of result */
				   "movl %%edx, %%ecx\n\t" /* and in this case, we should make division to the rest */
				   "1:\n\t"                /* or if we need to make only one division operation */
				   "movl %3, %%eax\n\t"
				   "movl %%ecx, %%edx\n\t"
				   "divl %%ebx\n\t"
				   "movl %%eax, %0\n\t"    /* lower 4 bytes of result */
				   "movl %%edx, %2\n"      /* rest indeed */
				   : "=g" (res_lp), "=g" (res_hp), "=g" (rst)
				   : "g" (ll_getlow(divid)), "g" (ll_gethigh(divid)), "b" (divis)
				   : "memory", "eax", "edx", "ecx");

  /***************************************************
	i don't know why, but simple code                *
	res = res_lp | (res_hp << 32);                   *
    doesn't work. so i use this assembler operation. *
    in this case it can be used just memmove.        *
                                                    */
  __asm__ volatile(""
				   : "=A" (res)
				   : "a" (res_lp), "d" (res_hp));

  if(neg)
	res = -res;	
  if(rest != NULL)
	*rest = rst;

  return res;
}

/* makes division witout the rest */
static inline int64_t ll_div(int64_t divid, int32_t divis)
{
  return ll_div_with_rest(divid, divis, NULL);
}

/* makes only mod operation */
static inline int32_t ll_mod(int64_t divid, int32_t divis)
{
  int32_t mod;

  ll_div_with_rest(divid, divis, &mod);
  return mod;
}

#endif /* __LL_DIV_H__ */
