/************************************************************************************\
 * - Basic functions and definitions for working with strigs                        *
 * - and some functions, wich work with memory.                                     *
 *                                                                                  *
 * Copyright(c) 2006 Dan Kruchinin (aka asgard)                                     *
 *                                                                                  *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     * 
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
\************************************************************************************/

#ifndef __STRING_H__
#define __STRING_H__

#include <klib/types.h>
#include <arch/types.h>

/* some ASCII specific defenitions for strcasestr function */
#define LCASE_START    98 /* ASCII code of first low case letter */
#define HCASE_START    66 /* ASCII code of first high case letter */
#define ASCII_NLETTERS 35 /* number of letters in ASCII */
#define HL_DIFF        32 /* difference between low case and high case letters */

static inline size_t strlen(const char* s)
{
  int32_t t0;
  register size_t __res;
  
  __asm__ volatile ("repne scasb\n\t"
					"notl %0\n\t"
					"decl %0\n"
					: "=c" (__res), "=&D" (t0)
					: "0" (0xFFFFFFFF), "1" (s), "a" (0));

  return __res;
}

static inline char* strncpy(char* str_to, const char* str_from, const size_t count)
{
  int32_t t0, t1, t2, t3;

  __asm__ volatile ("1:\n\t"
					"decl %2\n\t"
					"js 2f\n\t"
					"lodsb\n\t"
					"stosb\n\t"
					"testb %%al, %%al\n\t"
					"jne 1b\n\t"
					"2:\n\t"
					"movb $0, %%al\n\t"
					"stosb\n"
					: "=&S" (t0), "=&D" (t1), "=&c" (t2), "=&a" (t3)
					: "0" (str_from), "1" (str_to), "2" (count)
					: "memory");

  return str_to;
}

static inline char* strcpy(char* str_to, const char* str_from)
{
  return strncpy(str_to, str_from, strlen(str_from));
}

static inline char* strncat(char* str_to, const char* str_from, const size_t count)
{
  int32_t t0, t1, t2, t3;
  size_t len;

  len = strlen(str_to);

  __asm__ volatile ("1:\n\t"
					"decl %2\n\t"
					"js 2f\n\t"
					"lodsb\n\t"
					"stosb\n\t"
					"testb %%al, %%al\n\t"
					"jne 1b\n\t"
					"2:\n\t"
					"movb $0, %%al\n\t"
					"stosb\n"
					: "=&D" (t0), "=&S" (t1), "=&c" (t2), "=&a" (t3)
					: "0" (str_to + len), "1" (str_from), "2" (count)
					: "memory");

  return str_to;
}

static inline char* strcat(char* str_to, const char* str_from)
{
  return strncat(str_to, str_from, strlen(str_from));
}

static inline int32_t strncmp(const char* dest, const char* src, const size_t len)
{
  int32_t t0, t1, t2;
  register int32_t __res;

  __asm__ volatile ("1:\n\t"
					"decl %3\n\t"
					"js 2f\n\t"
					"lodsb\n\t"
					"scasb\n\t"
					"jg 3f\n\t"
					"jl 4f\n\t"
					"testb %%al, %%al\n\t"
					"jne 1b\n\t"
					"2:\n\t"
					"xorl %%eax, %%eax\n\t"
					"jmp end\n\t"
					"3:\n\t"
					"movl $1, %%eax\n\t"
					"jmp end\n\t"
					"4:\n\t"
					"movl $-1, %%eax\n\t"
					"end:\n"
					: "=a" (__res), "=&S" (t0), "=&D" (t1), "=&c" (t2)
					: "1" (dest), "2" (src), "3" (len));  
  
  return __res;
}

static inline int32_t strcmp(const char* dest, const char* src)
{
  return strncmp(dest, src, strlen(src));
}

static inline char* strchr(const char* str, const char c)
{
  int32_t t0;
  register char*  __ret;

  __asm__ volatile ("movb %%al, %%ah\n\t"
					"1:\n\t"
					"lodsb\n\t"
					"cmpb %%al, %%ah\n\t"
					"je 2f\n\t"
					"testb %%al, %%al\n\t"
					"jne 1b\n\t"
					"movl $0, %1\n\t"
					"jmp end\n\t"
					"2:\n\t"
					"decl %0\n\t"
					"movl %0, %1\n\t"
					"end:\n"
					: "=&S" (t0), "=a" (__ret)
					: "0" (str), "1" (c));  
  
  return __ret;
}

static inline char* strstr(const char* dest, const char* pattern)
{
  size_t len1, len2;
  register char* __ret;
  
  len1 = strlen(dest);
  len2 = strlen(pattern);

  __asm__ volatile ("movl $0, %%ebx\n\t"
					"1:\n\t"
					"cmpl %1, %%ebx\n\t"
					"je 3f\n\t"
					"movl %4, %%ecx\n\t"
					"movl %2, %%edi\n\t"
					"movl %3, %%esi\n\t"
					"addl %%ebx, %%edi\n\t"
					"rep cmpsb\n\t"
					"je 2f\n\t"
					"incl %%ebx\n\t"
					"jmp 1b\n\t"
					"2:\n\t"
					"movl %%edi, %%eax\n\t"
					"subl %4, %%eax\n\t"
					"jmp 4f\n\t"
					"3:\n\t"
   					"movl $0, %%eax\n\t"
					"4:\n"
					: "=a" (__ret) 
					: "g" (len1), "g" (dest), "g" (pattern), "g" (len2), "0" (0)
					: "%ebx");  
  
  return __ret;
}

/* fast memset realisation. fills 4 bytes during 1 cycle iteration */
static inline void* memset(void* dest, int32_t c, size_t size)
{
  int32_t t0, t1;

  __asm__ volatile ("movl %4, %%ecx\n\t"
					"sarl $0x02, %%ecx\n\t" /* how long we can set by 4 bytes */
					"rep stosl\n\t"
					"movl %4, %%ecx\n\t"
					"andl $0x03, %%ecx\n\t"
					"jz 1f\n\t"
					"rep stosb\n\t"
					"1:\n"
   	 				: "=&a" (t0), "=&D" (t1)
					: "0" (c), "1" (dest), "g" (size)
					: "memory");  

  return dest;  
}

/* fast memmove realisation. copies 4 bytes during 1 cycle iteration */
static inline void* memmove(void* dest, const void* src, size_t size)
{
  int32_t t0, t1;

  __asm__ volatile ("movl %4, %%ecx\n\t"
					"sarl $0x02, %%ecx\n\t"
					"rep movsl\n\t"
					"movl %4, %%ecx\n\t"
					"andl $0x03, %%ecx\n\t"
					"jz 1f\n\t"
					"rep movsb\n\t"
					"1:"
					: "=&D" (t0), "=&S" (t1)
					: "0" (dest), "1" (src), "g" (size)
					: "memory");

  return dest;
}

#endif /* __STRING_H__ */
