/*
 * JP (Jari Protocol) for internal and external,
 * kernel and/or server communication.
 * This is a part of the Jari microkernel.
 * 
 * Cmd subpart.
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <string.h>

#include <jp/cmd.h>
#include <kipc/errno.h>

/*general functions*/
int check_cmd_header(cmd_header *p)
{
  if(p->flag!=CMD_FLAG)
    return FAIL;

  return SUCCESS;
}

int create_cmd_header(cmd_header *p,char cmd,int nargs)
{
  if(cmd==0x00 && nargs<0)
    return FAIL;

  p->flag=CMD_FLAG;
  p->cmd=cmd;
  p->nargs=nargs; 
  p->args_offset=((sizeof(struct __cmd_header))+(2*nargs)*(sizeof(unsigned long))); 
  p->args_length=0; 
  p->aoff=malloc(nargs*(sizeof(unsigned long))); 
  p->alen=malloc(nargs*(sizeof(unsigned long)));

  if(!p->aoff && !p->alen)
    return FAIL; 

  return SUCCESS; 
}

int create_cmd_frame(cmd_header *p,cmd_frame *f,
		     void *data,unsigned long *lns)
{
  unsigned long gl=0,b=0;
  unsigned long *l=lns;
  unsigned long *e=NULL;
  unsigned long *o=NULL;
  void *d;

  /*header full in*/
  if(check_cmd_header(p)!=SUCCESS)
    return JP_INVALIDHDR;

  f->head=*p;

  if(p->nargs>0) {
    if(!lns && !data)
      return FAIL;
    
    while(*l!=0) {
      b+=*l;
      *l++;
    }

    p->args_length=b;
    l=lns;
    e=p->alen;
    o=p->aoff;
    *o=p->args_offset;
    /*fill in lengths and offsets*/
    while(*l!=0) {
      *o+=gl;
      *e=*l;
      gl=*l;
      *e++; *l++;
      *o++;
    }
  } else {
    f->data=NULL;
    return SUCCESS;
  }

  f->data=(void *)malloc((sizeof(void))*b);
  if(!f->data)
    return FAIL;

  memcpy(f->data,(const void*)data,b);

  return SUCCESS;
}

int reject_cmd_frame(cmd_frame *p)
{
  if(check_cmd_header(&(p->head))!=SUCCESS)
    return JP_INVALIDHDR;

  if(p->head.nargs>0) {
    free(p->head.aoff);
    free(p->head.alen);
    free(p->data);
  }
  free(p);

  return SUCCESS;
}

int check_cmd_frame(cmd_frame *p)
{
  if(check_cmd_header(&(p->head))!=SUCCESS)
    return JP_INVALIDHDR;

  if(p->head.nargs>0) {
    if(!p->data)
      return FAIL;
    if(!p->head.aoff && !p->head.alen)
      return FAIL;
  } else {
    if(p->data)
      return FAIL;
    if(p->head.aoff && p->head.alen)
      return FAIL;
  }

  return SUCCESS;
}

/*global args functions*/
void *get_cmd_whole_frame_data_ptr(cmd_frame *p)
{
  if(check_cmd_header(&(p->head))!=SUCCESS)
    return NULL;

  if(p->head.nargs==0)
    return NULL;
  else
    return p->data;

  return NULL;
}

int get_cmd_whole_frame_data(cmd_frame *p,void *data)
{
  void *h=get_cmd_whole_frame_data_ptr(p);

  if(h==NULL)
    return FAIL;
  else 
    memcpy(h,(const void*)p->data,p->head.args_length);

  return SUCCESS;
}

unsigned long get_ipc_whole_frame_data_length(cmd_frame *p)
{
  if(check_cmd_header(&(p->head))!=SUCCESS)
    return 0;

  if(p->head.nargs>0)
    return p->head.args_length;

  return 0;
}

unsigned long get_ipc_whole_frame_data_offset(cmd_frame *p)
{
  if(check_cmd_header(&(p->head))!=SUCCESS)
    return 0;

  if(p->head.nargs>0)
    return p->head.args_offset;

  return 0;
}

/*arguments functions*/
unsigned long get_cmd_frame_arg_length(cmd_frame *p,int narg)
{
  int d=1;
  unsigned long *e=NULL;

  if(check_cmd_header(&(p->head))!=SUCCESS)
    return 0;

  if(p->head.nargs<narg)
    return 0;
  else {
    if(check_cmd_frame(p)!=SUCCESS)
      return 0;

    e=p->head.alen;
    while(d!=narg) {
      d++;
      *e++;
    }
    return *e;
  }

  return 0;
}

void *get_cmd_frame_arg_ptr(cmd_frame *p,int narg)
{
  int d=1;
  void *w=NULL;
  unsigned long *o=NULL;

  if(check_cmd_header(&(p->head))!=SUCCESS)
    return NULL;

  if(p->head.nargs<narg)
    return NULL;
  else {
    if(check_cmd_frame(p)!=SUCCESS)
      return NULL;

    w=p->data;
    o=p->head.aoff;
    while(d!=narg) {
      d++;
      *o++;
      w+=*o;
    }

  }
  return w;

  return NULL;
}

int get_cmd_frame_arg(cmd_frame *p,void *arg,int narg)
{
  void *l=get_cmd_frame_arg_ptr(p,narg);
  unsigned long r=0;

  if(l==NULL)
    return FAIL;

  if((r=get_cmd_frame_arg_length(p,narg))==0)
    return FAIL;

  memcpy(arg,(const void*)l,r);

  return SUCCESS;
}

