/*
 * JP (Jari Protocol) for internal and external,
 * kernel and/or server communication.
 * This is a part of the Jari microkernel.
 * 
 * IPC subpart.
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 * 
 */

#include <string.h>

#include <jp/ipc.h>
#include <kipc/errno.h>

int check_ipc_header(ipc_header *p)
{
  if(p->flag!=IPC_FLAG)
    return FAIL;
  return SUCCESS;
}

int create_ipc_header(ipc_header *p)
{
  p->flag=IPC_FLAG;
  p->type=STD_FRAME;
  p->chunk_id=-1;
  p->trunk_id=-1;
  p->data_offset=sizeof(struct __ipc_header);
  p->data_length=0;
  return SUCCESS;
}

int change_ipc_frame_type(ipc_frame *p,char type)
{
  if(check_ipc_header(&(p->head))!=SUCCESS)
    return JP_INVALIDHDR;
  p->head.type=type;
  return SUCCESS;
}

int create_ipc_frame(ipc_header *p,ipc_frame *pt,
		     void *data,unsigned long length)
{
  if(check_ipc_header(p)!=SUCCESS)
    if(create_ipc_header(p)!=SUCCESS)
      return FAIL;
  if(!data && !length)
    return FAIL;
  if(!pt)
    return FAIL;
  p->data_length=length;
  p->data_offset+=sizeof(struct __ipc_frame);
  pt->head=*p;
  pt->data=data;
  return SUCCESS;
}

int reject_ipc_frame(ipc_frame *p)
{
  if(!p)
    return FAIL;
  if(check_ipc_header(&(p->head))!=SUCCESS)
    return JP_INVALIDHDR;
  if(!p->data)
    free(p->data);
  free(p);
  return SUCCESS;
}

void *get_ipc_frame_data_ptr(ipc_frame *p)
{
  if(p==NULL)
    return NULL;
  if(check_ipc_header(&(p->head))!=SUCCESS)
    return NULL;
  if(p->data!=NULL && p->head.data_length!=0)
    return p->data;
  else
    return NULL;

  return NULL;
}

int get_ipc_frame_data(ipc_frame *p,void *data)
{
  void *tp=NULL;

  if((tp=get_ipc_frame_data_ptr(p))==NULL)
    return FAIL;
  else {
    memcpy(data,(const void*)tp,p->head.data_length);
  }

  return SUCCESS;
}

unsigned long get_ipc_frame_data_length(ipc_frame *p)
{
  if(check_ipc_header(&(p->head))!=SUCCESS)
    return FAIL;

  return p->head.data_length;
}

unsigned long get_ipc_frame_data_offset(ipc_frame *p)
{
  if(check_ipc_header(&(p->head))!=SUCCESS)
    return FAIL;

  return p->head.data_offset;
}
