/*
 * Name servicing for kernel servers,
 * the part of the jari kernel.
 * Finally rewritten.
 *
 * Functions for working with registration of the servers.
 *
 * by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2001, 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <ns/ns.h>
#include <kipc/errno.h>
#include <arch/string.h> /*FIXME:*/

#include "nsroute.h"

/*internal troll pointers */
static srvchain *g_chain;

static int free_id=-1;

/*some predefined prototypes*/
static int _find_free_server_id(void);

/*general entrys*/
int init_nsserverschain(void)
{
  g_chain=NULL;

  g_chain=(srvchain *)kbmalloc(sizeof(struct __srvchain));
  if(g_chain==NULL)
    return FAIL;

  g_chain->next=NULL;
  g_chain->id=-127;

  return SUCCESS;
}

/*external functions*/

int reg_server(int priority, int type, long time,
	       char *name, void *data)
{
  int id=0;
  srvchain *y=g_chain;

  if(g_chain==NULL)
    return FAIL;

  if(type>=OUT_ILLEGAL_TYPE)
    return NS_REGINVALID;

  /*FIXME: check for exiting name in the list*/
  while(y->next!=NULL) {
    if(!strcmp(name,(const char*)y->name))
      return NS_SERVEREXITS;
    y=y->next;
  }
  y=g_chain;

  if(g_chain->id==-127) {
    g_chain->id=id;
    g_chain->type=type;
    g_chain->time=time;
    g_chain->data=data;
    g_chain->name=NULL;
    g_chain->name=(char *)kbmalloc(strlen(name)+1);
    if(g_chain->name==NULL)
      return FAIL;
    strcpy(g_chain->name,(const char*)name);
    g_chain->next=NULL;
  } else {
    if(free_id==-1) {
      id++;
      while(y->next!=NULL){
	id=y->id+1;
	y=y->next;
      }
    } else if(id>MAX_SERVERS) {
      id=-1;
      id=_find_free_server_id();
      if(id==-1)
	return NS_SERVERSLOTFULL;
    } else
      id=_find_free_server_id();
    y->next=(srvchain *) kbmalloc(sizeof(struct __srvchain));
    y=y->next;
    y->next=NULL;
    y->id=id;
    y->type=type;
    y->time=time;
    y->data=data;
    y->name=NULL;
    y->name=(char *)kbmalloc(strlen(name)+1);
    if(y->name==NULL)
      return FAIL;
    strcpy(g_chain->name,(const char*)name);
    y->next=NULL;
  }

  return SUCCESS;
}

int unreg_server(char *name)
{
  srvchain *prev=NULL;
  srvchain *curr=g_chain;
  int id=-1;

  id=get_server_id(name);
  if(id==-1)
    return NS_NOSUCHSERVER;
  if(_find_get_away_status(id)==FAIL)
    return NS_SERVERBUSY;

  free_id=id;

  while(curr!=NULL && curr->id!=id) {
    prev=curr;
    curr=curr->next;
  }
  if(curr=g_chain)
    g_chain=curr->next;
  else
    prev->next=curr->next;
  if(curr->data!=NULL)
    kbfree(data); /*how I mean we're need here to exec server wrapper?*/
  kbfree(curr->name);
  free(curr);

  return SUCCESS;
}

int get_server_id(char *name)
{
  srvchain *p=g_chain;

  while(p->next!=NULL) {
    if(!strcmp((const char*) name,(const char*)p->name))
      return p->id;
    p=p->next;
  }

  return -1;
}

int *find_server_by_type(int type)
{
  int *t=NULL;
  srvchain *c=g_chain;
  int *h;

  t=(int *)kbmalloc(MAX_SERVERS/4);
  if(!t)
    return FAIL;
  *t=-1;
  h=t;

  while(c->next!=NULL) {
    if(c->type==type) {
      *h=c->id;
      *h++;
      *h=-1;
    }
    c=c->next;
  }

  if(*t==-1)
    return NS_NOSUCHSERVER;

  return t;
}

static int _find_free_server_id(void)
{
  srvchain *p=g_chain;
  int f;

  if(free_id!=-1) {
    f=free_id;
    free_id=-1;
    return f;
  } else {
    while(p->next!=NULL) {
      /*FIXME:*/
      /*noop code*/
      p=p->next;
    }
  }

  return -1;
}

int get_srv_priority(int id) {
  srvchain *u=g_chain;

  while(u->next!=NULL) {
    if(id==u->id)
      return u->priority;
    u=u->next;
  }

  return NS_NOSUCHSERVER;
}

