/*
 * Name servicing for kernel servers,
 * the part of the jari kernel.
 * Finally rewritten.
 *
 * Functions for working with servers routing.
 *
 * by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2001, 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <ns/ns.h>
#include <kipc/errno.h>
#include <arch/string.h> /*FIXME:*/

static paths_route *proute;

static int free_id=-1;

static int _calc_priority_sum(nsroute *p);
static int _find_free_route_id(void);
static int _free_nsroute(nsroute *p);

int init_nspathsroute(void)
{
  proute=NULL;

  proute=(paths_route *)kbmalloc(sizeof(struct __routing));
  if(proute==NULL) 
    return FAIL;
  
  proute->next=NULL;
  proute->id=-127;

  return SUCCESS;
}

/*global functions*/
int add_nsroute(nsroute *p) 
{
  int id=0;
  paths_route *y=proute;

  if(proute==NULL)
    return FAIL;
  if(p==NULL)
    return FAIL;

  if(proute->id==-127) {
    proute->id=id;
    proute->priority=_calc_priority_sum(p);
    proute->pathroute=p;
    proute->next=NULL;
  } else {
    if(free_id==-1) {
      id++;
      while(y->next!=NULL){
	id=y->id+1;
	y=y->next;
      }
    } else if(id>MAX_PATHS) {
      id=-1;
      id=_find_free_route_id();
      if(id==-1)
	return NS_PATHSSLOTFULL;
    } else
      id=_find_free_route_id();
    y->next=(paths_route *) kbmalloc(sizeof(struct __routing));
    y=y->next;
    y->next=NULL;
    y->id=id;
    y->priority=_calc_priority_sum(p);
    y->pathroute=p;
  }

  return SUCCESS;
}

nsroute *get_routeid_by_ptr(nsroute *p)
{
  paths_route *y=proute;

  while(y->next!=NULL) {
    if(*p==*(y->pathroute))
      return y->id;
    y=y->next;
  }

  return FAIL;
}

int del_nsroute(nsroute *p)
{
  return SUCCESS;
}

int del_nsroute_by_id(int id)
{
  paths_route *prev=NULL;
  paths_route *curr=proute;

  if(id<0)
    return NS_NOSUCHROUTE;

  free_id=id;

  while(curr!=NULL && curr->id!=id) {
    prev=curr;
    curr=curr->next;
  }
  if(curr=proute)
    proute=curr->next;
  else
    prev->next=curr->next;
  if(curr->pathroute!=NULL)
    _free_nsroute(curr->pathroute);
  kbfree(curr->name);
  free(curr);

  return SUCCESS;
}

int get_high_nsroute(void)
{
  int hdi=0;
  return hdi;
}

int _find_get_away_status(int id)
{
  paths_route *y=proute;
  nsroute *t=NULL;

  while(y->next!=NULL) {
    t=y->pathroute;
    if(t!=NULL)
      while(t->next!=NULL) {
	if(id==t->id)
	  return FAIL;
	t=t->next;
      }
    y=y->next;
  }

  return SUCCESS;
}

/*local functions*/
static int _calc_priority_sum(nsroute *p)
{
  int sum=0;
  int g=0,ts=-1;

  if(!p)
    return FAIL;

  while(p->next!=NULL) {
    ts=get_srv_priority(p->id);
    if(ts!=NS_NOSUCHSERVER)
      sum+=ts;
    else
      return FAIL;
    g++;
    p=p-next;
  }

  return sum/g;
}

static int _find_free_route_id(void)
{
  paths_route *p=proute;
  int f;

  if(free_id!=-1) {
    f=free_id;
    free_id=-1;
    return f;
  } else {
    while(p->next!=NULL) {
      /*FIXME:*/
      /*noop code*/
      p=p->next;
    }
  }

  return -1;
}

static int _free_nsroute(nsroute *p)
{
  nsroute *prev=NULL;
  nsroute *curr=p;

  if(!p)
    return FAIL;

  while(curr!=NULL) {
    if(curr==p)
      p=curr->next;
    kbfree(curr);
    curr=p;
  }

  return SUCCESS;
}

