/*
 * Virtual file system implementation.
 * Point(directory) operations.
 * The part of the Jari microkernel.
 *
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <perm.h>
#include <vfs/vfs.h>
#include <vfs/types.h>
#include <kipc/errno.h>

int vfs_mkpnt(char *name)
{
  super_fnode *super=NULL;
  fnode *node=NULL;
  fnode *nnode=NULL;
  dpoint *parent=NULL;
  char *tn=name;
  int _s=0;
  char *root=NULL;
  char *a=NULL;

  if(!(super=lookup_superfs_fnode(tn)))
    return FAIL;
  if(!(node=(fnode*)malloc(sizeof(struct __fnode)))) {
    free(super);
    return FAIL;
  }
  _s=strlen(super->mpoint);
  while(_s!=0) {
    *tn++;
    _s--;
  }
  root=tn;
  a=tn;
  /*get the needable name*/
  while(*a!='\0')
    *a++;
  while(*a!=PNT_SEP)
    *a--;

 ret_read:
  while(*tn!=PNT_SEP) {
    *tn++;
  }
  *tn='\0';
  if(!(node=super->def_func->get_fnode(root))) {
    free(super);
    return FAIL;
  } else {
    super->def_func->read_fnode(node);
    if(node->type!=VFS_POINT) {
      free(super);
      free(node);
      return FAIL;
    } else {
      *tn=PNT_SEP;
      if(strcmp((const char*)tn,(const char*)a))
	goto ret_read;
    }
  }
  *a++;  /*seek*/

  /*create the point*/
  if(!(nnode=(fnode*)malloc(sizeof(struct __fnode)))) {
    free(super);
    free(node);
    return FAIL;
  }
  nnode->name=a;
  super->def_func->put_fnode(nnode); /*put fnode*/
  /*okay, let's read parent*/
  *tn--;/*seek*/
  while(*tn!=PNT_SEP)
    *tn--;
  *tn='\0';

  super->def_func->get_fnode(root);
  super->def_func->read_fnode(node);

  if(!(parent=(dpoint*)malloc(sizeof(struct __dpoint)))) {
    super->def_func->clear_fnode(nnode); /*delete fnode*/
    nnode=super->def_func->unset_fnode(nnode);
    free(nnode);
    free(node);
    free(super);
    return FAIL;
  }
  node->def_func->read_pnt(node,NULL,parent);
  *tn=PNT_SEP;

  super->def_func->write_fnode(nnode);
  nnode->def_func->mkpnt(nnode,parent);

  free(nnode);
  free(parent);
  free(super);
  free(node);

  return SUCCESS;
}

int vfs_rmpnt(char *name)
{
  super_fnode *super=NULL;
  fnode *node=NULL;
  dpoint *path=NULL;
  char *tn=name;
  char *root=NULL;
  int _s=0;

  if(!(super=lookup_superfs_fnode(name)))
    return FAIL; 

  if(!(node=(fnode *)malloc(sizeof(struct __fnode)))) {
    free(super);
    return FAIL;
  } else {
    _s=strlen(super->mpoint);
    while(_s!=0) {
      _s--;
      *tn++;
    }
    root=tn;

    node=super->def_func->get_fnode(root);
    super->def_func->read_fnode(node);
    if(node->type!=VFS_POINT) {
      free(super);
      free(node);
      return FAIL;
    }
  }
  node->def_func->read_pnt(node,NULL,path);
  if(path->pchild || path->fchild || !path->parent) {
    free(super);
    free(node);
    free(path);
    return FAIL;
  } else {
    node->def_func->rmpnt(node,path);
    node=super->def_func->unset_fnode(node);
  }

  free(path);
  free(node);
  free(super);

  return SUCCESS;
}

int vfs_read_pnt(char *name, dpoint *data)
{
  super_fnode *super=NULL;
  fnode *node=NULL;
  char *tn=name;
  char *root=NULL;
  int _s=0;

  if(!(super=lookup_superfs_fnode(name)))
    return FAIL;
  if(!(node=(fnode *)malloc(sizeof(struct __fnode)))) {
    free(super);
    return FAIL;
  } else {
    _s=strlen(super->mpoint);
    while(_s!=0) {
      _s--;
      *tn++;
    }
    root=tn;

    node=super->def_func->get_fnode(root);
    super->def_func->read_fnode(node);
    if(node->type!=VFS_POINT) {
      free(node);
      free(super);
      return FAIL;
    } else
      node->def_func->read_pnt(node,NULL,data);
  }

  free(super);
  free(node);

  return SUCCESS;
}

