/*
 * Virtual file system implementation.
 * SuperFileNode operations.
 * The part of the Jari microkernel.
 *
 * Written by Alfeiks Kaanoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include <perm.h>
#include <vfs/types.h>
#include <vfs/vfs.h>
#include <kipc/errno.h>

int vfs_stat_mpoint(void *mpoint,void *data)
{
  return SUCCESS;
}

fnode *_get_fnode(char *name)
{
  fnode *p=NULL;
  super_fnode *ss=lookup_superfs_fnode((char *)name);

  if(!ss)
    return p;
  p=(fnode *)malloc(sizeof(struct __super_fnode));
  //p=ss->def_func->get_fnode(p,data);

  free(ss);

  return p;
}

fnode *_unset_fnode(super_fnode *pp,fnode *p)
{
  if(!pp && !p)
    return p;

  p=pp->def_func->unset_fnode(p);

  if(p)
    free(p);
  return NULL;
}

int _read_fnode(super_fnode *pp,fnode *p)
{
  if(!pp && !p)
    return FAIL;

  pp->def_func->read_fnode(p);

  return SUCCESS;
}

int _write_fnode(super_fnode *pp,fnode *p)
{
  if(!pp && !p)
    return FAIL;

  pp->def_func->write_fnode(p);

  return SUCCESS;
}

int _put_fnode(super_fnode *pp,fnode *p)
{
  if(!pp && !p)
    return FAIL;

  pp->def_func->put_fnode(p);

  return SUCCESS;
}

int _clear_fnode(super_fnode *pp,fnode *p)
{
  if(!pp && !p)
    return FAIL;

  pp->def_func->clear_fnode(p);

  return SUCCESS;
}

