/*
 * Minimal(Mountable) FileSystem, The part of the Jari
 * kernel.
 *
 * Originally written by Alfeiks Kaänoken 
 * <akaanoken@gmail.com> (c) 2004, 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 * Structures and functions prototypes.
 *
 */

#ifndef __MFS__MFS_H__
#define __MFS__MFS_H__

/*
 * The file system support under 16Tb partition and
 * 2Gb file size. It's made for caching VFS and
 * paralleling purposes, not for strong data
 * storage purposes.
 * -- version 0.0.1 (2004) by Alfeiks K.
 * MRK: 128 unicode chars for file name (ILIX)
 */

/*
 * The filesystem supports under 4Gb partition,
 * and made *only* for image or memory usage.
 * Maximal lenght of the file name is 128 chars,
 * or 128 16bit chars in the Jari (unicode).
 * -- version 0.0.2 (2006) by Alfeiks K.
 */

/*#define ILIX*/

#ifndef __ILIX__
#include <perm.h>
#else
#include <ilix/acl.h>
#endif

/*
 * Minimal File System has the following structure.
 * H->block_size:blocks_reg*Nblock:Reserved for file allocation
 * percent of the free blocks:FileSystem data.
 * were Nblock is a total number of the blocks on the fs
 * H is the header of the filesystem. 
 */

/*blocks sizes*/
#define LOW_SIZE     1024 /*only for compatibility*/
#define MEDIUM_SIZE  2048 /*default for 0.0.2 it's optimal*/
#define HIGH_SIZE    4096 /*default for 0.0.1 for big blobs*/

#define RESERVED_PERCENT  10 /* reserved percent of the blocks for fs structure
			      * storage. Originally was 5, but I changed this.
			      */

#define MAX_FILE_BLOCKS 16384

#define MFS_VERSION  "0.0.2"
#define MFS_NAME     "MFS"
#define MINOR_REV    1
#define MAJOR_REV    1

#define SYS_LINUX  0xB0
#define SYS_JARI   0xA0
#define SYS_XBSD   0xC0
#define SYS_SUN    0xD0

#define B_EMPTY     0x1
#define B_BLOB      0xA
#define B_SYSTEM    0xD
#define B_RESERVED  0xF

#define VFS_REGULAR    1
#define VFS_POINT      2
#define VFS_MEMORY     3
#define VFS_DEVICE     4
#define VFS_SERVER     5
#define VFS_PIPE       6
#define VFS_INTERFACE  7
#define VFS_SOCKET     8
#define VFS_BUFFER     9
#define VFS_CPU        10
#define VFS_HOST       11
#define VFS_NODE       12
#define VFS_FS         13
#define VFS_VIDEO      14
#define VFS_CUSTOM     15
#define VFS_LINK       16

typedef struct __mfs_head_t {
  char version[10];
  char system;
  int block_size;
  unsigned long free_blocks;
  unsigned long total_data_blocks;
  int revision[2];
  time_t last_checked;
} mfs_head_t;

typedef struct __mfs_block_t {
#ifdef __ILIX__
  int space_addr : 8;
#endif
  unsigned long block_addr;
  int state; /*empty,blob,system,reserved*/
  int fill_in : 4; /*filling of the block*/
} mfs_block_t;

typedef struct __mfs_inode_t {
  char file_name[128];
  mfs_block_t *blocks; 
  time_t last_access;
  time_t last_modification;
  time_t created;
  char type;
#ifndef __ILIX__
  perm_t permission;
#else
  acl_t permission;
#endif
#ifdef __ILIX__
  node_t node_owner;
  parm_t paralized_ticks;
  parm_t seek_ticks;
  cache_t persistent;
#endif
  struct __mfs_inode_t *next;
  struct __mfs_inode_t *parent;
  struct __mfs_inode_t **child;
} mfs_inode_t;

typedef struct __mfs_dir_t {
  mfs_inode_t *owner;
  mfs_inode_t **child;
} mfs_dir_t;

#endif /*__MFS__MFS_H__*/
