/*
 * YacFS, Yet Another Contributed FileSystem.
 * The prefferred file system for the Jari Operating System.
 *
 * Originally written by Alfeiks Kaänoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#ifndef __WRIO_FILE__H__
#define __WROI_FILE__H__

#include <yacfs.h>
#include <yacfs_logic.h>

/*API for the file i/o with the YacFS image*/

/*
 * Opening an image,
 * if no error occured 0 is returned, otherwise -1 will be returning.
 */
int open_yacfs_image(char *fname);

/*
 * Returning a pointer to the head of the YacFS
 * image or NULL if error occured.
 */
yacfs_head_t *get_yacfs_image_head(void);

/*
 * Reads inode header by it's number.
 * Returns 0 if succeful.
 */
int _read_yacfs_image_inode_n(yacfs_inode_t *inode, unsigned long long n);

/*
 * Write inode header by it's number.
 * Returns 0 if all is ok.
 */
int _write_yacfs_image_inode_n(yacfs_inode_t *inode, unsigned long long n);

/*
 * Fill up whole the yacfs_inode structure and reads it by it's number.
 * NOTE: it's copy data to the structure for the caching purposes.
 */
int read_yacfs_image_inode_n(yacfs_inode *inode,unsigned long long n); /*COPY!!*/

/*
 * Write up the whole yacfs_inode structure by it's number.
 */
int write_yacfs_image_inode_n(yacfs_inode *inode,unsigned long long n);

/*
 * absolute read, this function for debug only, or for small files.
 * Data must be allocated.
 */
int read_yacfs_image_data_b(yacfs_inode *inode,void *data,unsigned long size);

/*
 * absolute write, this functions rewrite all the data.
 * Use with carefully.
 */
int write_yacfs_image_data_b(yacfs_inode *inode,void *data,unsigned long size);

/* There functions for working with file marker
 * the read function cannot override the data,
 * but write is can.
 */
int read_yacfs_image_data(yacfs_inode *inode,void *data,unsigned long size,
			  unsigned long seek);

int write_yacfs_image_data(yacfs_inode *inode,void *data,unsigned long size,
			   unsigned long seek);


#endif /*__WRIO_FILE__H__*/

