/*
 * YacFS, Yet Another Contributed FileSystem.
 * The prefferred file system for the Jari Operating System.
 *
 * Originally written by Alfeiks Kaänoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#ifndef __YACFS_H__
#define __YACFS_H__

/*
 * version 0.0.1pa, the initial implementation
 * nov 2006, AK
 */

/*the magic of the header, starts on the beggining YACFS space*/
#define YACFS_MAGIC  0x01ddc6fc

/*defines for the filetypes*/
#define YACFS_REGULAR    0x01                                                        
#define YACFS_POINT      0x02                                                        
#define YACFS_MEMORY     0x03                                                        
#define YACFS_DEVICE     0x04                                                        
#define YACFS_SERVER     0x05                                                        
#define YACFS_PIPE       0x06                                                        
#define YACFS_INTERFACE  0x07                                                        
#define YACFS_SOCKET     0x08                                                        
#define YACFS_BUFFER     0x09                                                        
#define YACFS_CPU        0x10                                                       
#define YACFS_HOST       0x11                                                       
#define YACFS_NODE       0x12                                                       
#define YACFS_FS         0x13                                                       
#define YACFS_VIDEO      0x14                                                       
#define YACFS_CUSTOM     0x15                                                       
#define YACFS_LINK       0x16

/*
 * yacfs is designed to be simply and flexibility.
 * header contain varios info, such as number of the 
 * free and busy inodes, state of the flushing data,
 * offset of the begging data.
 *
 */

#define YACFS_FLASHED  1
#define YACFS_NONFLASHED  0

/* this type is used for the addressing space, 
 * actually we're can addressing up to 4Tb, on the
 * spaces less than 2Gb we're don't use a primary key.
 */
typedef struct __yacfs_offset_t {
  unsigned long primary;
  unsigned long secondary;
} yacfs_offset_t;

#define YACFS_JARI   0xee
#define YACFS_LINUX  0xfa
#define YACFS_SOLAR  0xaf
#define YACFS_XXBSD  0xf0

typedef struct __yacfs_head_t {
  unsigned int reserved_percent;
  unsigned long long free_inodes;
  unsigned long long busy_inodes;
  unsigned long long space_size;
  unsigned long long blocks;
  unsigned long long used_blocks;
  unsigned int flashed : 2;
  int inode_size;
  int block_size;
  int inode_per_blocks;
  int version;
  char system;
#ifdef __SHARED_SET__
  yacfs_offset_t data_segment;
#endif /*__SHARED_SET__*/
} yacfs_head_t;

#define YACFS_BLOCKFREE   0
#define YACFS_BLOCKDIRTY  1
#define YACFS_BLOCKFULL   2
#define YACFS_BLOCKINDIR  4

typedef struct __yacfs_block_t {
  int state;
  int fill_in;
  unsigned int p_blocks; /*this info needs for big data*/
  yacfs_offset_t offset; /*current block offset ont the space*/
  yacfs_offset_t next_block; /*offsets of the pointed blocks*/
} yacfs_block_t;

/*
 * inode can contains data,
 * or just links to other inodes.
 * actually this is a block (the set of the inodes)
 * inode contains all hiearchy infos
 */

#define YACFS_INODEFREE    0
#define YACFS_INODEFULL    1
#define YACFS_INODEMASTER  2
#define YACFS_INODESLAVE   3
#define YACFS_INODEDIRTY   4

/*
 * The following structure we're have.
 * Inode head (yacfs_inode_t)
 * name
 * childs inodes.
 * blocks of the data.
 *
 */

typedef struct __yacfs_inode_t {
  unsigned long long inode_number;
  yacfs_offset_t m_offset;
  int state; /*state of the inode*/
  /*there is an indexing information*/
  unsigned long path_hash; 
  unsigned long full_name_hash;
  /*information on inodes*/
  int owner_id;
  int group_id;
  int node_id;
  int ogn[3];
  char type;
  int name_len;
  /*hiearchy information*/
  unsigned long long inode_parent;
  unsigned int child_num;
  /*data information*/
  unsigned long long inode_continios; /*the inode that contain continuing information*/
  unsigned int affected_blocks;
} yacfs_inode_t;

#endif /*__YACFS_H__*/
