/*
 * YacFS, Yet Another Contributed FileSystem.
 * The prefferred file system for the Jari Operating System.
 *
 * Originally written by Alfeiks Kaänoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

#ifndef __YACFS_LOGIC__H_
#define __YACFS_LOGIC__H_

#include <yacfs.h>

#define YACFS_NOSPACE  -2
#define YACFS_READERR  -4
#define YACFS_MEMERR   -1
#define YACFS_WRERR    -12
#define YACFS_UNKNERR  -9

/*
 * Therminology of the file systems in the Jari
 * doesn't have matches with the linux or 
 * other *nix therminology.
 */

typedef struct __yacfs_inode {
  yacfs_inode_t inode;
  char *name;
  yacfs_inode_t *childs;
  yacfs_block_t *blk;
  yacfs_offset_t *blocks;
} yacfs_inode;

typedef struct __yacfs_super_block {
  yacfs_inode *inode;
  yacfs_block_t *blocks;
  yacfs_offset_t *blocks_offsets;
} yacfs_super_block;

/*functions of the data storage*/
unsigned long q_hash(const char *name);

/*the inode functions*/
int get_inode(char *fname,yacfs_inode_t parent,yacfs_inode_t *inode);
int read_inode(yacfs_inode *inode,yacfs_inode_t parent);
int write_inode(yacfs_inode *inode,yacfs_inode_t parent);
int put_inode(yacfs_inode_t parent,yacfs_inode *inode);
int unlink_inode(yacfs_inode_t parent,yacfs_inode *inode);
int clear_inode(yacfs_inode_t parent,yacfs_inode *inode);

#endif /*__YAC_FS_LOGIC__H_*/
