/*
 * YacFS, Yet Another Contributed FileSystem.
 * The prefferred file system for the Jari Operating System.
 * mkfs.yacfs an utility for the linux to formatting space by
 * YacFS.
 *
 * Originally written by Alfeiks Kaänoken <akaanoken@gmail.com>
 * (c) 2006
 *
 * This software is free software; you can redistribute it and/or                
 * modify it under the terms of the GNU Lesser General Public                   
 * License as published by the Free Software Foundation; either                 
 * version 2 of the License, or (at your option) any later version.             
 *                                                                              
 * This software is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU            
 * Lesser General Public License for more details.                              
 *                                                                              
 * You should have received a copy of the GNU Lesser General Public             
 * License along with this library; if not, write to the                        
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,                 
 * Boston, MA 02111-1307, USA. 
 *
 */

/* 
 * mkfs.yacfs version 0.0.1a (prealpha),
 * without reserved space formatting
 * AK, 2006 nov.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>

#include <yacfs.h>

#define STD_BLOCKSIZE   2048
#define STD_INODESIZE   4096
#define STD_RESPERCENT  10

const int revision=0;
const int major=0;
const int minor=1;
const char *mod="prealpha";

static yacfs_head_t header;

unsigned long long dspc=0;

void _print_version(void);

int _init_header(int fd)
{
  struct stat st;
  unsigned long long sp_size=0;
  unsigned long long abs_size=0;
  unsigned long long super_blocks=0;
  unsigned long long blocks=0;
  unsigned long long raw_data_size=0;
  unsigned long magic=YACFS_MAGIC;
  unsigned long h_s=sizeof(YACFS_MAGIC)+sizeof(struct __yacfs_head_t);
  unsigned long long super_size=header.inode_size;

  if(fstat(fd,&st)) {
    fprintf(stderr,"%s\n",strerror(errno));
    return -1;
  }

  dspc=st.st_size;
  lseek(fd,0,SEEK_SET);
  sp_size=st.st_size;
  abs_size=sp_size-h_s; /*determine the size*/
  super_size+=(header.block_size+sizeof(struct __yacfs_block_t))*header.inode_per_blocks;
  super_blocks=abs_size/super_size;
  if((abs_size%super_size) != 0 && super_size*super_blocks > abs_size) {
    super_blocks--;
  }

  blocks=super_blocks*header.inode_per_blocks;
  raw_data_size=blocks*header.block_size;

#if 0
  /*debug*/
  fprintf(stderr,"Super blocks(inodes): %lld\nBlocks: %lld\nSpace for raw data: %lld bytes.\n",
	  super_blocks,blocks,raw_data_size);
  fprintf(stderr,"Super block size: %lld bytes.\n",super_size);
  fprintf(stderr,"Absolute size: %lld bytes.\n",abs_size);
  fflush(stderr);
  /*end of debug*/
#endif
  /*so, let's init header the stage two*/
  header.free_inodes=super_blocks-1; /*reserving for root inode*/
  header.busy_inodes=1; /*one is reserved for root*/
  header.space_size=abs_size;
  header.blocks=blocks;
  header.used_blocks=0;
  /*let's write this*/
  if(!write(fd,(const void*)&magic,sizeof(YACFS_MAGIC))) {
    fprintf(stderr,"%s\n",strerror(errno));
    return -1;
  } 
  lseek(fd,(off_t)sizeof(YACFS_MAGIC),SEEK_SET); /*TODO: check this*/
  if(!write(fd,(const void*)&header,sizeof(struct __yacfs_head_t))) {
    fprintf(stderr,"%s\n",strerror(errno));
    return -1;
  }

  _print_version();
  fflush(stdout);
  fprintf(stdout,"Formatting yacfs with the following parameters:\n"\
	  "Inode size: %d bytes.\nBlock size:%d bytes\n"		\
	  "Density of inodes: %d per blocks.\nTotal space: %lld bytes.\n",
	  header.inode_size,header.block_size,header.inode_per_blocks,
	  abs_size);
  fflush(stdout);

  return 0;
}

void _set_inode_offset(unsigned long long c, yacfs_offset_t *o)
{
  unsigned long long full_addr=0;
  unsigned long h_size=sizeof(YACFS_MAGIC)+sizeof(struct __yacfs_head_t);

  /*let's calculate absolute address*/
  full_addr=h_size + (c*(header.inode_size+(header.inode_per_blocks*(header.block_size+sizeof(struct __yacfs_block_t)))));

  o->primary=full_addr&0xffffffff; /*FIXME: make a better this operation*/
  o->secondary=full_addr >> 32;

  return;
}

void _set_null_block(unsigned long long c, int a, yacfs_block_t *b)
{
  unsigned long long full_addr=0;
  unsigned long long super_addr=0;
  unsigned long h_size=sizeof(YACFS_MAGIC)+sizeof(struct __yacfs_head_t);
  int bl=sizeof(struct __yacfs_block_t);

  /*calculate addresses*/
  super_addr=h_size + (c*(header.inode_size+(header.inode_per_blocks*(header.block_size+bl))));
  full_addr=super_addr + header.inode_size + bl + (a*(bl+header.block_size));

  b->offset.primary=full_addr&0xffffffff; /*FIXME: make a better this operation*/
  b->offset.secondary=full_addr >> 32;
  b->state=YACFS_BLOCKFREE;
  b->fill_in=0;
  b->p_blocks=0;

  return;
}

void _set_null_node(yacfs_inode_t *i,unsigned long long count)
{
  yacfs_offset_t offt;

  i->inode_number=count;
  _set_inode_offset(count,&offt);
  i->m_offset=offt;
  i->state=YACFS_INODEFREE;
  i->path_hash=0;
  i->full_name_hash=0;
  i->owner_id=0;
  i->group_id=0;
  i->node_id=0;
  i->ogn[0]=0; i->ogn[1]=0; i->ogn[2]=0;
  i->type=YACFS_REGULAR;
  i->name_len=0;
  i->inode_parent=0;
  i->child_num=0;
  i->inode_continios=0;
  i->affected_blocks=0;

  return;
}

int _format_space(int fd)
{
  unsigned long long super_count=0;
  unsigned long long super_blocks=header.busy_inodes + header.free_inodes;
  yacfs_inode_t inode;
  yacfs_block_t block;
  unsigned long super_size=0;
  void *buf=NULL;
  void *t=NULL;
  int i;

  unsigned long long per=10*(super_blocks/100);
  int g=0;

  super_size=header.inode_size+(header.inode_per_blocks*(header.inode_size+sizeof(struct __yacfs_block_t)));
  if(!(buf=calloc(1,super_size))) {
    fprintf(stderr,"Error allocating space for buffer.\n");
    return -1;
  }
  t=buf;

  fprintf(stdout,"Formatting space:");
  fflush(stdout);

  /*TODO: make a check*/
  lseek(fd,(off_t)(sizeof(YACFS_MAGIC)+sizeof(struct __yacfs_head_t)),SEEK_SET);

#if 0
  /*debug*/
  fprintf(stderr,"\nSuper_blocks=%lld.\n:",super_blocks);
  fflush(stderr);
  /*end debug*/
#endif

  while(super_count!=super_blocks) {
    /*put inode*/
    _set_null_node(&inode,super_count);
    memcpy(t,(const void*)&inode,sizeof(struct __yacfs_inode_t));
    t+=header.inode_size;

    /*put blocks*/
    for(i=0; i < header.inode_per_blocks; i++) {
      _set_null_block(super_count,i,&block);
      memcpy(t,(const void*)&block,sizeof(struct __yacfs_block_t));
      t+=sizeof(struct __yacfs_block_t);
      t+=header.block_size;
    }

    /*write this super block*/
    /*TODO: make a check*/
    write(fd,(const void*)buf,super_size);
    //lseek(fd,(off_t)super_size,SEEK_CUR);

    super_count++;
    fdatasync(fd);
    t=buf;

    if(super_count>per*g) {
      g++;
      fprintf(stdout,"=");
      fflush(stdout);
    }
  }

  /*TODO: make a check*/
  lseek(fd,(off_t)(sizeof(YACFS_MAGIC)+sizeof(struct __yacfs_head_t)),SEEK_SET);
  /*put root node*/
  _set_null_node(&inode,0);
  inode.ogn[0]=7;
  inode.ogn[1]=7;
  inode.ogn[2]=7;
  inode.state=YACFS_INODEDIRTY;
  inode.type=YACFS_POINT;
  write(fd,(const void*)&inode,sizeof(struct __yacfs_inode_t));
  fdatasync(fd);
  lseek(fd,0,SEEK_SET);

  fprintf(stdout,":\nDone.\n");
  fflush(stdout);

  free(buf);

  return 0;
}

void _print_version(void)
{
  fprintf(stdout,"Jari tools: mkfs.yacfs version %d.%d.%d%s.\n"\
	  "Written by Alfeiks Kaanoken <akaanoken@gmail.com> (c) 2006\n",
	  revision,major,minor,mod);
  return;
}

void _print_usage(void)
{
  fprintf(stdout,"Usage:\n"\
	  "mkfs.yacfs <OPTIONS> file.\n"\
	  "OPTIONS:\n"\
	  "\t-b <value> : Block size in bytes, possible values is 1024,2048,4096.\n"\
	  "\t-i <value> : Inode size in bytes, possible values is 4096,8192,16384.\n"\
	  "\t-d <value> : Inode per block, use with carefully. default: 4\n"\
	  "\t-r <value> : Percent of the reserved space in blocks. default: 10\n"\
	  "\t-v : Show version info.\n"					\
	  "\t-h : Print this help.\n");
  return;
}

int main(int argc, char **argv)
{
  int _block_size=STD_BLOCKSIZE;
  int _inode_size=STD_INODESIZE;
  int _inode_per_block=4;
  int _reserved_percent=10;
  char *file_name=NULL;
  char *buf=NULL;
  int i=0;
  int ter=0;
  int fd=-1;

  if(argc < 2) {
    _print_version();
    _print_usage();
    return 0;
  }

  /*parse options*/
  **argv++;
  argc--;
  while(i!=argc) {
    buf=*argv;

    if(*buf=='-' && strlen((const char*)buf)>1) {
      if(strlen((const char*)buf) > 2) 
	goto unknown_option;      
      *buf++;

      switch(*buf) {
      case 'b':
	**argv++; i++;
	if(!(buf=*argv)) {
	  fprintf(stderr,"No parameter to 'b' given.\n");
	  _print_usage();
	  return 0;
	} else {
	  ter=atoi(buf);
	  if(ter!=1024 && ter!=2048 && ter!=4096) {
	    fprintf(stderr,"Invalid parameter to 'b' given.\n");
	    _print_usage();
	    return 0;
	  } else 
	    _block_size=ter;
	}
	break;
      case 'i':
	**argv++; i++;
	if(!(buf=*argv)) {
	  fprintf(stderr,"No parameter to 'i' given.\n");
	  _print_usage();
	  return 0;
	} else {
	  ter=atoi(buf);
	  if(ter!=4096 && ter!=8192 && ter !=16384) {
	    fprintf(stderr,"Invalid parameter to 'i' given.\n");
	    _print_usage();
	    return 0;
	  } else
	    _inode_size=ter;
	}
	break;
      case 'd':
	**argv++; i++;
	if(!(buf=*argv)) {
	  fprintf(stderr,"No parameter to 'd' given.\n");
	  _print_usage();
	  return 0;
	} else {
	  ter=atoi(buf);
	  if(ter < 4 || ter > 2048) {
	    fprintf(stderr,"Invalid parameter to 'd' given.\nMust be in (4-2048).\n");
	    _print_usage();
	    return 0;
	  } else
	    _inode_per_block=ter;
	}
	break;
      case 'r':
	**argv++; i++;
	if(!(buf=*argv)) {
	  fprintf(stderr,"No parameter to 'r' given.\n");
	  _print_usage();
	  return 0;
	} else {
	  ter=atoi(buf);
	  if(ter < 0 || ter > 80) {
	    fprintf(stderr,"Invalid parameter to 'r' given.\nMust be in (0-80).\n");
	    _print_usage();
	    return 0;
	  } else
	    _reserved_percent=ter;
	}
	break;
      case 'v':
	_print_version();
	break;
      case 'h':
	_print_usage();
	break;
      default:
      unknown_option:
	fprintf(stderr,"Unknown option '%s' given.\n",buf);
	_print_usage();
	return 0;
	break;
      }
    } else if(*buf=='-') {
      fprintf(stderr,"No option given after '-'.\n");
      _print_usage();
      return 0;
    }

    file_name=buf;
    **argv++; i++;
  }
  /*so, now we will init the header, stage one*/
  header.reserved_percent=_reserved_percent;
  header.version=1;
  header.system=YACFS_LINUX;
  header.inode_size=_inode_size;
  header.inode_per_blocks=_inode_per_block;
  header.block_size=_block_size;
  header.flashed=YACFS_FLASHED;
  /*debug*/
#if 0
  fprintf(stderr,"YACFS header stage1:\n"\
	  "Reserved of blocks: %d\n"\
	  "Version: %d\n"	     \
	  "Inode size: %d bytes.\n"  \
	  "Block size: %d bytes.\n"  \
	  "Inode per blocks: %d.\n",
	  header.reserved_percent,header.version,header.inode_size,
	  header.block_size,header.inode_per_blocks);
  fprintf(stderr,"File space to format %s.\n",file_name);
#endif
  /*end debug*/
  if((fd=open((const char*)file_name,O_RDWR|O_EXCL))!=-1) {
    if(_init_header(fd)!=0)
      goto end_off;
    if(_format_space(fd)!=0)
      goto end_off;
  } else {
    fprintf(stderr,"%s\n",strerror(errno));
    return 0;
  }

 end_off:
  close(fd);

  return 0;
}

